/**
  * @file pwm_driver_pic32.c
  * @brief This module contains the implementation of the PWM driver hardware
  * function

  * @author Stefan Popovic
  *
  * @date 11/09/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: P32MX795F512L
  * Dev.Board: EasyPIC Fusion v7
  * Oscillator: HS-PLL 80 MHz
  * Ext. Modules: PWM driver click Board
  * SW: MikroC PRO for PIC32 v3.6.0

  * NOTES:
    - Place PWM driver click board on mikroBUS slot 1
    - Turn on PORTD LEDs at SW15, pull-down PORTB.0 and PORTB.1 and
      turn on Button Press Level switch SW10 for PORTB/L (board specific)
    - PWM: RD0  Channel 1, prescaler 1, 16 - bit
    - ADC: 10 - bit

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/

  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/

  /**************************************************************************
  * Module Typedefs
  **************************************************************************/

  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "built_in.h"
  #include "resources.h"
  #include <stdint.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
  uint16_t current_duty;
  uint16_t pwm_period1;
  uint16_t adc_result;
  char txt[12];

  // TFT module connections
  char TFT_DataPort at LATE;
  sbit TFT_RST at LATD7_bit;
  sbit TFT_BLED at LATD2_bit;
  sbit TFT_RS at LATD9_bit;
  sbit TFT_CS at LATD10_bit;
  sbit TFT_RD at LATD5_bit;
  sbit TFT_WR at LATD4_bit;
  char TFT_DataPort_Direction at TRISE;
  sbit TFT_RST_Direction at TRISD7_bit;
  sbit TFT_BLED_Direction at TRISD2_bit;
  sbit TFT_RS_Direction at TRISD9_bit;
  sbit TFT_CS_Direction at TRISD10_bit;
  sbit TFT_RD_Direction at TRISD5_bit;
  sbit TFT_WR_Direction at TRISD4_bit;
  // End TFT module connections

  void MCU_Init()
  {
    TRISB0_bit = 1;                      //configure PORTB pin 0 as input
    TRISC1_bit = 0;                      //configure PORTC pin 1 as output (PWM)
    JTAGEN_bit = 0;
   
    ADC1_Init();                         // ADC Init
    Delay_ms(100);
    current_duty = 0;                    // initial value for current duty

    PWM_Init(5000 , 1, 1, 2);            // PWM Initialization
    PWM_Start(1);
    PWM_Set_Duty(current_duty,  1);      // Set current duty for PWM1
   }
   
   void Display_Init()
   {
    TFT_BLED_Direction = 0;
    TFT_Init_ILI9341_8bit(320, 240);     // TFT Init
    TFT_BLED = 1;
   }

   void DrawFrame()
   {
    TFT_Set_Pen(CL_WHITE, 1);
    TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
    TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_Black, 1);
    TFT_Line(20, 220, 300, 220);
    TFT_LIne(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("PWM driver click", 90, 14);
    TFT_Write_Text("Duty cycle:", 40, 100);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("%", 220, 100);
    TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("EasyPIC Fusion v7 for P32MX", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
   }
 
  void main() 
  {
    MCU_Init();
    Display_Init();
    DrawFrame();

    while( 1 ) 
    {
      uint16_t cur_duty_byte;
    
      // Read 10 - bit ADC value and set newly acquired 8 - bit PWM duty
      adc_result = ADC1_Read(0); 
      current_duty = adc_result; 
      PWM_Set_Duty(adc_result * 4, 1);       // Set newly acquired duty
      cur_duty_byte = current_duty;
      cur_duty_byte = cur_duty_byte * 0.098; // 100 / 1024
      WordToStr(cur_duty_byte, txt);
      TFT_Set_Pen(CL_WHITE, 1);
      TFT_Rectangle(145, 100, 200, 120);
      TFT_Set_Pen(CL_BLACK, 2);
      TFT_Write_Text(txt, 150, 100);
      Delay_ms(20);
    }
  }